/* closure to wrap a desktop dropdown menu
 *
 * settings contain
 * parent: jQuery Div
 * id: id of the dropdown-group
 * labelText: 
 * titleText: the button's default value
 * listValues: a JSON-Object with visible values as keys and internal representations values to the respective key
 * width: css property
 * [onChange]: event listener that is called if something changes
   [onOpen]: event listener to notify the owner that the dialoge is opened
   [marginBottom]
 *
 * */
function createDropdowndesktop(settings)
{
	
	var mContainer;
	var mLabelContainer;
	var mDropdownContainer;
	var mMenu;
	var mButton;
	var mButtonText;
	var mLastValue="";
	var mLabel;
	var mLabelValue;
	var mHint;
	
	inflate();
	setupCSS();
	
	function inflate()
	{
		mContainer=jQuery("<div/>").addClass("dropdown-container");
		mContainer.css("width",settings.width);
		
		mLabelContainer=jQuery("<div/>").addClass("dropdown-label-container");
		mLabelContainer.addClass("d-none d-md-block");
		mDropdownContainer=jQuery("<div/>").addClass("dropdown-dropdown-container");
		mContainer.append(mLabelContainer);
		mContainer.append(mDropdownContainer);
		
		mLabel=jQuery("<label/>").text(settings.labelText);
		
		mLabelValue=jQuery("<label/>").text(settings.labelText);//second label for mobile portrait view
		mLabelValue.addClass("d-block d-md-none");
		mDropdownContainer.append(mLabelValue);
		
		var dropdown=jQuery("<div/>").addClass("dropdown");
		mButton=jQuery("<button/>").addClass("btn text-left dropdown-toggle");
		mButton.attr("id",settings.id+"-button");
		mButton.attr("type","button").attr("data-bs-toggle","dropdown");
		mButton.on("click",onOpen);
		
		mButtonText=jQuery("<span/>").text(settings.titleText);
		mButtonText.attr("data-text",settings.titleText);		
		mButton.append(mButtonText);
		
		//var icon=jQuery("<img/>").attr("src","https://my-pergola24.elaspix.de/elaspix/konfigurator/pics/dark_hds.svg");
		//icon.css("width","12px").css("position","absolute").css("right","0px");
		//icon.css("margin","8px").css("margin-right","12px");
		mMenu=jQuery("<ul/>").addClass("dropdown-menu w-100");
		
		
		mLabelContainer.append(mLabel);
		mDropdownContainer.append(dropdown);
		dropdown.append(mButton);
		//mButton.append(icon);
		dropdown.append(mMenu);
		//mGroupsDiv.append(group);
		
		mHint=jQuery("<small/>").text(settings.hintText);
		mDropdownContainer.append(mHint);
		settings.parent.append(mContainer);
		//var groupStructure={"group":mContainer,"button":button,"buttonText":buttonText,"menu":menu,"onChange":settings.onChange,"lastValue":""};
		//mDropDownGroups[aId]=groupStructure;
		setValues(settings.listValues);
	}
	
	function setValues(aValues)
	{		
			
			mMenu.empty();			
			jQuery.each(Object.keys(aValues),function(index,key){
				
				var entry=jQuery("<li/>").append(jQuery("<a/>").addClass("dropdown-item parametervalue").text(key).attr("href","#"));
				entry.on("click",function(){onValueSelect(key);});
				entry.css("cursor","pointer").css("user-select","none");
				entry.css("word-wrap","break-word");
				entry.css("overflow","hidden");			
				mMenu.append(entry);			
			}); 
	}
	
	function setTitle(aTitle)
	{
		mButtonText.text(aTitle);
		mButtonText.attr("data-text",aTitle);	
		mLastValue=aTitle;//to allow selection of the same value as before
	}
	
	function onOpen()
	{
		if (settings.onOpen!=undefined)
		{
			settings.onOpen();
		}
	}
	
	function onValueSelect(aKey)
	{
		internalValue=settings.listValues[aKey];
		console.log("onValueSelect with Value "+internalValue);
		mButtonText.text(aKey);
		mButtonText.attr("data-text",internalValue);
		//mLabelValue.text(settings.labelText+" "+aKey);
		
		if ((mLastValue!=internalValue) && (settings.onChange!=undefined))
		{
			mLastValue=internalValue;//only fire if the value is new
			settings.onChange(internalValue,settings.labelText);
		}else{
			clearHint();
		}
	}
	
	function setupCSS()
	{
		mContainer.addClass("mt-3");
		mLabel.addClass("parameter");
		mLabel.css("padding",".2rem .75rem");
		mLabelValue.addClass("parameter");
		mButtonText.addClass("parametervalue");
		
		mContainer.css("display","flex");
		mLabelContainer.css("width","50%");
		//mLabelContainer.css("background-color","blue");
		mDropdownContainer.css("width","50%");
		//mDropdownContainer.css("background-color","red");
		
		mHint.addClass("form-text note");//note is used in stead of text-muted
		
		mButton.addClass("btn-dropdown");//style now handled in css section
		// mButton.css("border-radius",".25rem");//default bootstrap is overwritten by elementor
		mButton.css("color","#333333");
		mButton.css("border","none");
		mButton.css("background-color","white");//default bootstrap is overwritten by elementor
		mButton.hover(
				function(){mButton.css("background-color","white");},//take all all hover effects from elementor
				function(){mButton.css("background-color","white");}
				);
		mButton.css("letter-spacing","normal");
		if (settings.marginBottom)
		{
			mContainer.css("margin-bottom",settings.marginBottom);
		}
		
	}
	
	function getValue()
	{
		return mButtonText.attr("data-text");
	}
	
	function selectValue(aValue){
		var key;
		jQuery.each(settings.listValues, function(k, value){
			if(aValue == value){
				key = k;
			}
		});
		
		mButtonText.text(key);
		mButtonText.attr("data-text", aValue);
		mLastValue = aValue;
		
	}
	
	function setLabelText(aText){
		mLabel.text(aText);
	}
	
	function setHintText(aText)
	{
		mHint.text(aText);
	}
	
	function setHintTextHTML(aHTML)
	{
		mHint.html(aHTML);
	}
	
		//aHintObjects is a list of tuples with hint:text and click:function-Properties
	function setHint(aHintObjects)
	{
		mHint.empty();
		jQuery.each(aHintObjects,function(index,hintObject){
					var span=jQuery("<span/>").text(hintObject.hint);
					if (hintObject.hasOwnProperty("click"))
					{
						span.css("text-decoration","underline").css("cursor","pointer");
						span.on("click",hintObject.click);
					}
					
					if (hintObject.hasOwnProperty("cssClass"))
					{
						span.addClass(hintObject.cssClass);
					}
					mHint.append(span);
				});
	}
	
	function setHintClass(aClass)
	{
		mHint.removeClass();
		mHint.addClass("form-text");//padding
		
		mHint.addClass(aClass);
		if (aClass!="text-danger" && aClass!="text-warning" && aClass!="text-success")
		{//show gray color only if text-danger is not given which forces to show text in red
			mHint.addClass("text-muted");//gray
		}
	
	}
	
	function clearHint()
	{
		mHint.text("");
		setHintClass("");//uses text-muted
	}
	
	function fadeOut()
	{
		mContainer.fadeOut("slow",function(){mContainer.hide("slow");});		
	}
	
	function hide()
	{
		mContainer.css("display","none");
	}
	
	function show()
	{
		//mContainer.show("fast");
		mContainer.css("display","flex");
	}
	
	function getLabelText()
	{
		return mLabel.text();
	}
	
	return {
		"setValues":setValues,
		"getValue":getValue,
		"setTitle":setTitle,
		"selectValue":selectValue,
		"setLabelText":setLabelText,
		"getLabelText":getLabelText,
		"setHintText":setHintText,
		"setHintTextHTML":setHintTextHTML,
		"setHint":setHint,//replaces the html content and appends the passed jQuery-Object
		"clearHint":clearHint,
		"setHintClass":setHintClass,
		"hide":hide,
		"fadeOut":fadeOut,
		"show":show
	};
	
	
}
